@@ -28,6 +28,7 @@
 
 #include <config.h>
 
+#include <glib/gi18n.h>
 
 #include <libgnome/libgnome.h>
 #include <libgnomeui/libgnomeui.h>
@@ -157,6 +158,9 @@ display_contributors (gpointer data)
 	AnimationData *ani_data;
 
 	static GnomeCanvasItem *contributor = NULL;
+	/* FIXME: this is a workaround for a libgnomecanvas bug. See bug #329165.
+	 * When removing it, we should not forget to remove the "ugh" part of
+	 * animate_contributor() */
 	static GnomeCanvasItem *contributor_rect = NULL;
 	static GnomeCanvasItem *contributor_text = NULL;
 
@@ -295,6 +299,7 @@ display_introduction_message (gpointer data)
 	AnimationData *ani_data;
 
 	static GnomeCanvasItem *intro = NULL;
+	/* FIXME: this is a workaround for a libgnomecanvas bug. See bug #329165. */
 	static GnomeCanvasItem *intro_rect = NULL;
 	static GnomeCanvasItem *intro_text = NULL;
 	static gint intro_i = 0;
@@ -416,9 +421,10 @@ load_random_header (void)
 	struct dirent *d;
 
 	directory = gnome_program_locate_file (NULL,
-					       GNOME_FILE_DOMAIN_DATADIR,
-					       "gnome-about/headers/",
+					       GNOME_FILE_DOMAIN_APP_DATADIR,
+					       "headers",
 					       TRUE, NULL);
+
 	if (!directory) {
 		show_error_dialog (_("Could not locate the directory with header images."));
 
@@ -476,18 +482,18 @@ load_random_header (void)
 }
 
 static GdkPixbuf *
-load_button (void)
+load_logo (void)
 {
 	gchar *file;
 	GdkPixbuf *pixbuf;
 	GError *error = NULL;
 
 	file = gnome_program_locate_file (NULL,
-					  GNOME_FILE_DOMAIN_DATADIR,
-					  "gnome-about/gnome-64.gif",
+					  GNOME_FILE_DOMAIN_APP_DATADIR,
+					  "gnome-64.png",
 					  TRUE, NULL);
 	if (!file) {
-		show_error_dialog (_("Could not locate the GNOME logo button."));
+		show_error_dialog (_("Could not locate the GNOME logo."));
 
 		return NULL;
 	}
@@ -666,27 +672,26 @@ get_description_messages (xmlNodePtr node)
 {
 	xmlNodePtr paras;
 	GSList *list = NULL, *l;
-	const GList *langs = NULL;
+	const char * const * langs;
 	gint i;
 	gboolean started = FALSE;
 	char *best_value = NULL;
 
-	langs = gnome_i18n_get_language_list ("LC_MESSAGES");
+	langs = g_get_language_names ();
 	paras = node->children;
 
 	while (paras) {
 		while (paras) {
-			char *value = (char *)xmlNodeGetContent (paras);
+			xmlChar *value = xmlNodeGetContent (paras);
 			char *tmp;
 			xmlChar *cur, *best = NULL;
-			const GList *t;
 
 			if (paras->type == XML_ELEMENT_NODE &&
-					xmlStrEqual (paras->name, "p") &&
+					xmlStrEqual (paras->name, (const xmlChar *) "p") &&
 					value && value[0]) {
 				cur = xmlNodeGetLang (paras);
 
-				tmp = strip_newlines (value);
+				tmp = strip_newlines ((const gchar *) value);
 
 				if (!started) {
 					started = TRUE;
@@ -699,17 +704,19 @@ get_description_messages (xmlNodePtr node)
 					best_value = g_strdup (tmp);
 				}
 				else {
-					if (!cur || xmlStrEqual (cur, "C")) {
+					guint i;
+
+					if (!cur || xmlStrEqual (cur, (const xmlChar *) "C")) {
 						break;
 					}
 					/* See if the current lanaguage occurs
 					 * earlier than the previous best. */
-					for (t = langs; t; t = t->next) {
-						if (xmlStrEqual (t->data,
+					for (i = 0; langs[i] != NULL; ++i) {
+						if (xmlStrEqual ((const xmlChar *) langs[i],
 								best)) {
 							break;
 						}
-						else if (xmlStrEqual (t->data,
+						else if (xmlStrEqual ((const xmlChar *) langs[i],
 									cur)) {
 							xmlFree (best);
 							best = xmlStrdup (cur);
@@ -721,7 +728,7 @@ get_description_messages (xmlNodePtr node)
 							 * stop. We are not
 							 * going to go any
 							 * better. */
-							if (t == langs) {
+							if (i == 0) {
 								break;
 							}
 						}
@@ -776,45 +783,39 @@ create_date_string (const char *value)
 	return result;
 }
 
-static void
-display_version_info (GnomeCanvasGroup *group)
+static gboolean
+get_version_info (char **version_string,
+		  char **distributor_string,
+		  char **build_date_string)
 {
+	gchar *file;
+
 	xmlDocPtr about;
 	xmlNodePtr node;
 	xmlNodePtr bits;
 
-	gchar *file;
-
 	char *platform = NULL;
 	char *minor = NULL;
 	char *micro = NULL;
-	char *version_string = NULL;
-	char *distributor_string = NULL;
-	char *build_date_string = NULL;
-	char *text = NULL;
-
-	GnomeCanvasItem *info;
-	gdouble height = 0.0;
 
 	file = gnome_program_locate_file (NULL,
-					  GNOME_FILE_DOMAIN_DATADIR,
-					  "gnome-about/gnome-version.xml",
+					  GNOME_FILE_DOMAIN_APP_DATADIR,
+					  "gnome-version.xml",
 					  TRUE, NULL);
-	if (!file) {
-		show_error_dialog (_("Could not locate the file with GNOME version information."));
-	}
+	if (!file)
+		return FALSE;
 
 	about = xmlParseFile (file);
 	g_free (file);
 
 	if (!about)
-		return;
+		return FALSE;
 
 	node = about->children;
 
-	if (g_ascii_strcasecmp (node->name, "gnome-version")) {
+	if (g_ascii_strcasecmp ((const char *) node->name, "gnome-version")) {
 		xmlFreeDoc (about);
-		return;
+		return FALSE;
 	}
 
 	bits = node->children;
@@ -831,14 +832,14 @@ display_version_info (GnomeCanvasGroup *group)
 		if (!g_ascii_strcasecmp (name, "platform")
 		    && value && value[0])
 			platform = g_strdup (value);
-		if (!g_ascii_strcasecmp (name, "minor") && value && value[0])
+		else if (!g_ascii_strcasecmp (name, "minor") && value && value[0])
 			minor = g_strdup (value);
-		if (!g_ascii_strcasecmp (name, "micro") && value && value[0])
+		else if (!g_ascii_strcasecmp (name, "micro") && value && value[0])
 			micro = g_strdup (value);
-		if (!g_ascii_strcasecmp (name, "distributor") && value && value[0])
-			distributor_string = g_strdup (value);
-		if (!g_ascii_strcasecmp (name, "date") && value && value[0])
-			build_date_string = create_date_string (value);
+		else if (!g_ascii_strcasecmp (name, "distributor") && value && value[0])
+			*distributor_string = g_strdup (value);
+		else if (!g_ascii_strcasecmp (name, "date") && value && value[0])
+			*build_date_string = create_date_string (value);
 
 		bits = bits->next;
 		xmlFree (value);
@@ -847,29 +848,76 @@ display_version_info (GnomeCanvasGroup *group)
 	xmlFreeDoc (about);
 
 	if (!minor)
-		version_string = g_strconcat (platform, NULL);
+		*version_string = g_strdup (platform);
 
-	if (!version_string && !micro)
-		version_string = g_strconcat (platform, ".", minor, NULL);
+	if (!*version_string && !micro)
+		*version_string = g_strconcat (platform, ".", minor, NULL);
 
-	if (!version_string)
-		version_string = g_strconcat (platform, ".", minor, ".",
-					      micro, NULL);
+	if (!*version_string)
+		*version_string = g_strconcat (platform, ".", minor, ".",
+					       micro, NULL);
 
 	g_free (platform);
 	g_free (minor);
 	g_free (micro);
 
+	return TRUE;
+}
+
+static gboolean
+display_version_info_on_term (void)
+{
+	char *version_string = NULL;
+	char *distributor_string = NULL;
+	char *build_date_string = NULL;
+
+	if (!get_version_info (&version_string,
+			       &distributor_string,
+			       &build_date_string)) {
+		g_printerr (_("Could not get information about GNOME version."));
+		return FALSE;
+	}
+
+	g_print (_("%s: %s\n"), _("Version"), version_string);
+	g_print (_("%s: %s\n"), _("Distributor"), distributor_string);
+	g_print (_("%s: %s\n"), _("Build Date"), build_date_string);
+		 
+	g_free (version_string);
+	g_free (distributor_string);
+	g_free (build_date_string);
+
+	return TRUE;
+}
+
+static void
+display_version_info (GnomeCanvasGroup *group)
+{
+	char *version_string = NULL;
+	char *distributor_string = NULL;
+	char *build_date_string = NULL;
+	char *format = NULL;
+	char *text = NULL;
+
+	GnomeCanvasItem *info;
+	gdouble height = 0.0;
+
+	if (!get_version_info (&version_string,
+			       &distributor_string,
+			       &build_date_string))
+		show_error_dialog (_("Could not get information about GNOME version."));
+
 	info = gnome_canvas_item_new (group,
 				      gnome_canvas_group_get_type (),
 				      "x", 10.0,
 				      NULL);
 
+	format = g_strdup_printf ("<b>%s</b>%%s", _("%s: "));
+
 	if (version_string && version_string[0]) {
 		gdouble tmp;
 		GnomeCanvasItem *item;
 
-		text = g_strdup_printf ("<b>%s: </b>%s",
+		text = g_strdup_printf (format,
 					_("Version"), version_string);
 		item = gnome_canvas_item_new (GNOME_CANVAS_GROUP (info),
 					      gnome_canvas_text_get_type (),
@@ -888,7 +936,7 @@ display_version_info (GnomeCanvasGroup *group)
 		gdouble tmp;
 		GnomeCanvasItem *item;
 
-		text = g_strdup_printf ("<b>%s: </b>%s",
+		text = g_strdup_printf (format,
 					_("Distributor"), distributor_string);
 		item = gnome_canvas_item_new (GNOME_CANVAS_GROUP (info),
 					      gnome_canvas_text_get_type (),
@@ -907,7 +955,7 @@ display_version_info (GnomeCanvasGroup *group)
 		gdouble tmp;
 		GnomeCanvasItem *item;
 
-		text = g_strdup_printf ("<b>%s: </b>%s",
+		text = g_strdup_printf (format,
 					_("Build Date"), build_date_string);
 		item = gnome_canvas_item_new (GNOME_CANVAS_GROUP (info),
 					      gnome_canvas_text_get_type (),
@@ -922,6 +970,7 @@ display_version_info (GnomeCanvasGroup *group)
 		height += tmp + 4.0;
 	}
 
+	g_free (format);
 	g_free (version_string);
 	g_free (distributor_string);
 	g_free (build_date_string);
@@ -944,7 +993,7 @@ create_canvas (void)
 	GnomeCanvasPoints *points;
 
 	GdkPixbuf *header;
-	GdkPixbuf *button;
+	GdkPixbuf *logo;
 
 	gdouble current_x;
 	gdouble current_y;
@@ -979,14 +1028,14 @@ create_canvas (void)
 				      "pixbuf", header,
 				      NULL);
 
-	/* load button logo in advance, we need it's size */
-	button = load_button ();
-	if (!button)
+	/* load logo in advance, we need it's size */
+	logo = load_logo ();
+	if (!logo)
 		/* emergency stop complete with leaks */
 		return NULL;
 
 	/* and a coulple o' links */
-	current_x = 10.0 + (gdouble)gdk_pixbuf_get_width (button) + 10.0;
+	current_x = 10.0 + (gdouble)gdk_pixbuf_get_width (logo) + 10.0;
 	current_y = (gdouble)gdk_pixbuf_get_height (header) + 5.0;
 
 	href = href_item_new (root,
@@ -1059,12 +1108,12 @@ create_canvas (void)
 
 	gnome_canvas_points_free (points);
 
-	/* the gnome button logo */
+	/* the gnome logo */
 	item = gnome_canvas_item_new (root,
 				      gnome_canvas_pixbuf_get_type (),
 				      "x", 10.0,
 				      "y", 10.0,
-				      "pixbuf", button,
+				      "pixbuf", logo,
 				      NULL);
 
 	/* and some introduction text */
@@ -1201,22 +1250,42 @@ generate_randomness (void)
 	g_rand_free (generator);
 }
 		
+static gboolean gnome_version = FALSE;
+
+static const GOptionEntry options[] = {
+  { "gnome-version", 0, 0, G_OPTION_ARG_NONE, &gnome_version, N_("Display information on this GNOME version"), NULL },
+  { NULL }
+};
 
 /* main */
 int
 main (int argc, char **argv)
 {
+	GOptionContext *context;
+	GnomeProgram *program;
 	GtkWidget *dialog;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	gnome_program_init ("gnome-about", "1.0",
-			    LIBGNOMEUI_MODULE,
-			    argc, argv, NULL);
-	gnome_window_icon_set_default_from_file (GNOME_ICONDIR
-			"/gnome-logo-icon-transparent.png");
+	context = g_option_context_new (NULL);
+	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+
+	program = gnome_program_init ("gnome-about", VERSION,
+				      LIBGNOMEUI_MODULE,
+				      argc, argv,
+				      GNOME_PARAM_APP_DATADIR, DATADIR,
+				      GNOME_PARAM_GOPTION_CONTEXT, context,
+				      NULL);
+
+	if (gnome_version) {
+		g_object_unref (program);
+		return (display_version_info_on_term ()) ? 0 : 1;
+	}
+
+	gtk_window_set_default_icon_from_file (GNOME_ICONDIR
+			"/gnome-logo-icon-transparent.png", NULL);
 
 	dialog = create_about_dialog ();
 	if (!dialog)
@@ -1228,5 +1297,7 @@ main (int argc, char **argv)
 
 	gtk_main ();
 
+	g_object_unref (program);
+
 	return 0;
 }
